import os
import shutil

from setuptools import Extension, Distribution
import numpy
from Cython.Build import cythonize
from Cython.Distutils.build_ext import new_build_ext as cython_build_ext


def build() -> None:
    # if running in RTD, skip compilation
    if os.environ.get("READTHEDOCS") == "True":
        return

    # compile FLI library
    os.system("cd lib && make && cd ..")

    extensions = [
        Extension(
            "pyobs_fli.flidriver",
            ["pyobs_fli/flidriver.pyx"],
            library_dirs=["lib/"],
            libraries=["fli", "cfitsio"],
            include_dirs=[numpy.get_include()],
            extra_compile_args=["-fPIC"],
        )
    ]
    ext_modules = cythonize(extensions)

    distribution = Distribution(
        {
            "name": "extended",
            "ext_modules": ext_modules,
            "cmdclass": {
                "build_ext": cython_build_ext,
            },
        }
    )

    distribution.run_command("build_ext")

    # copy to source
    build_ext_cmd = distribution.get_command_obj("build_ext")
    for ext in build_ext_cmd.extensions:
        filename = build_ext_cmd.get_ext_filename(ext.name)
        shutil.copyfile(os.path.join(build_ext_cmd.build_lib, filename), filename)


if __name__ == "__main__":
    build()



import asyncio
import logging
from typing import Any, Optional, Union

from pyobs_fli.flidriver import DeviceType

log = logging.getLogger(__name__)


class FliBaseMixin:
    """A base class for pyobs module for FLI devices."""

    __module__ = "pyobs_fli"

    def __init__(
        self,
        dev_type: DeviceType,
        dev_name: Optional[str] = None,
        dev_path: Optional[str] = None,
        keep_alive_ping: int = 10,
        **kwargs: Any,
    ):
        """Initializes a new FliCamera.

        If neither dev_name nor dev_path are given, the first found device is used.

        Args:
            dev_type: Device type.
            dev_name: Optional name for device.
            dev_path: Optional path to device.
            keep_alive_ping: Interval for keep alive ping.
        """
        from .flidriver import FliDriver  # type: ignore

        # variables
        self._dev_type = dev_type
        self._dev_name = dev_name
        self._dev_path = dev_path
        self._keep_alive_ping = keep_alive_ping
        self._driver: Optional[FliDriver] = None
        self._device: Optional[Any] = None

        # keep alive
        self.add_background_task(self._keep_alive)

    async def open(self) -> None:
        """Open module."""
        from .flidriver import FliDriver

        # list devices
        devices = FliDriver.list_devices(self._dev_type)
        if len(devices) == 0:
            raise ValueError("No FLI device found.")

        # do we have a device name or path given?
        if self._dev_name is not None or self._dev_path is not None:
            # yes, find matching device
            for dev in devices:
                if dev.name.decode("utf-8") == self._dev_name or dev.filename.decode("utf-8") == self._dev_path:
                    self._device = dev
                    break
            else:
                raise ValueError("No matching device found, check dev_name/dev_path.")

        else:
            # no, just pick first one
            self._device = devices[0]

        # open driver
        log.info(
            'Opening connection to "%s" at %s...',
            self._device.name.decode("utf-8"),
            self._device.filename.decode("utf-8"),
        )
        self._driver = FliDriver(self._device)
        try:
            self._driver.open()
        except ValueError as e:
            raise ValueError("Could not open FLI camera: %s", e)

    async def close(self) -> None:
        # not open?
        if self._driver is not None:
            # close connection
            self._driver.close()
            self._driver = None

    async def _keep_alive(self) -> None:
        """Keep connection to camera alive."""
        from .flidriver import FliDriver

        while True:
            # is there a valid driver?
            if self._driver is not None:
                # then we should be able to call it
                try:
                    self._driver.get_serial_string()
                except ValueError:
                    # no? then reopen driver
                    log.warning("Lost connection to camera, reopening it.")
                    self._driver.close()
                    self._driver = FliDriver(self._device)

            await asyncio.sleep(self._keep_alive_ping)


__all__ = ["FliBaseMixin"]



import logging
from typing import Tuple, Any, Optional, Dict, List, Union
from itertools import chain

from pyobs.mixins import MotionStatusMixin
from pyobs.modules import Module
from pyobs.interfaces import IFilters, IFitsHeaderBefore
from pyobs.utils.enums import MotionStatus
from pyobs_fli.flibase import FliBaseMixin
from pyobs_fli.flidriver import DeviceType
import pyobs.utils.exceptions as exc

log = logging.getLogger(__name__)


class FliFilterWheel(FliBaseMixin, Module, MotionStatusMixin, IFilters, IFitsHeaderBefore):
    """A pyobs module for FLI filter wheels."""

    __module__ = "pyobs_fli"

    def __init__(self, filter_names: Union[List[str], List[List[str]]], **kwargs: Any):
        """Initializes a new FliFilterWheel.

        Args:
            filter_names: Names of filters.
        """
        Module.__init__(self, **kwargs)
        FliBaseMixin.__init__(self, dev_type=DeviceType.FILTERWHEEL, **kwargs)
        MotionStatusMixin.__init__(self, motion_status_interfaces=["IFilters"])

        # filter names, make it a list of lists, if not already is
        self._filter_names = filter_names
        if isinstance(self._filter_names[0], str):
            self._filter_names = [self._filter_names]

    async def open(self) -> None:
        """Open module."""
        await Module.open(self)
        await FliBaseMixin.open(self)

        # check
        if self._driver is None:
            raise ValueError("No driver found.")

        # serial number
        serial = self._driver.get_serial_string()
        log.info("Connected to filter wheel with serial number: %s", serial)

        # idle
        await self._change_motion_status(MotionStatus.IDLE)

    async def close(self) -> None:
        """Close the module."""
        await Module.close(self)
        await FliBaseMixin.close(self)

    async def list_filters(self, **kwargs: Any) -> List[str]:
        """List available filters.

        Returns:
            List of available filters.
        """
        return list(chain.from_iterable(self._filter_names))

    async def set_filter(self, filter_name: str, **kwargs: Any) -> None:
        """Set the current filter.

        Args:
            filter_name: Name of filter to set.

        Raises:
            ValueError: If an invalid filter was given.
            MoveError: If filter wheel cannot be moved.
        """

        # get filter pos and set it
        if filter_name in self._filter_names[0]:
            p = self._filter_names[0].index(filter_name)
            pos = 0 if p == 0 else 7 - p
        elif filter_name in self._filter_names[1]:
            p = self._filter_names[1].index(filter_name)
            pos = 7 * p
        else:
            raise exc.ModuleError("Filter not found")

        # move filter
        await self._change_motion_status(MotionStatus.SLEWING)
        self._driver.set_filter_pos(pos)
        await self._change_motion_status(MotionStatus.POSITIONED)

    async def get_filter(self, **kwargs: Any) -> str:
        """Get currently set filter.

        Returns:
            Name of currently set filter.
        """

        # get filter pos and return filter name
        div, mod = divmod(self._driver.get_filter_pos(), 7)
        if mod == 0:
            return self._filter_names[1][div]
        else:
            return self._filter_names[0][0 if mod == 0 else 7 - mod]

    async def init(self, **kwargs: Any) -> None:
        """Initialize device.

        Raises:
            InitError: If device could not be initialized.
        """
        pass

    async def park(self, **kwargs: Any) -> None:
        """Park device.

        Raises:
            ParkError: If device could not be parked.
        """
        pass

    async def stop_motion(self, device: Optional[str] = None, **kwargs: Any) -> None:
        """Stop the motion.

        Args:
            device: Name of device to stop, or None for all.
        """
        pass

    async def is_ready(self, **kwargs: Any) -> bool:
        """Returns the device is "ready", whatever that means for the specific device.

        Returns:
            Whether device is ready
        """
        return True

    async def get_fits_header_before(
        self, namespaces: Optional[List[str]] = None, **kwargs: Any
    ) -> Dict[str, Tuple[Any, str]]:
        """Returns FITS header for the current status of this module.

        Args:
            namespaces: If given, only return FITS headers for the given namespaces.

        Returns:
            Dictionary containing FITS headers.
        """
        return {"FILTER": (await self.get_filter(), "Current filter")}


__all__ = ["FliFilterWheel"]
